# Script name: Breeders equation
#
# Authors: Jon Bancic
#
# Date Created: 2023-01-23
#
# This script provides a function to look at the effects of 
# changing the components of breeders equation and a few other 
# parameters.
#
# Create scenarios by changing the values of one parameter at 
# the time.

# ---- Clean environment and load packages ----

rm(list = ls())
# install.packages(pkgs = "AlphaSimR")
packageDescription(pkg = "AlphaSimR")$Version
library(package = "AlphaSimR")

# ---- Load in plotting and simulation function ----
plotOutput <- function(data, variable, ylab = NULL, legendTitle = NULL) {
  # Required packages
  require(ggplot2)
  require(dplyr)
  options(dplyr.summarise.inform = FALSE)
  # Plot
  nReps <-  max(data$rep)
  data %>%
    group_by(cycle, Scenario) %>%
    summarise(Mean = mean(.data[[variable]]),
              SD = sd(.data[[variable]], na.rm = T) / sqrt(nReps)) %>%
    ggplot(aes(x = cycle, y = Mean, color = Scenario, fill = Scenario)) +
    geom_ribbon(aes(ymin = Mean - SD, ymax = Mean + SD), alpha = .3, colour = NA) +
    geom_line() +
    theme_bw() +
    xlab("Breeding Cycle") +
    ylab(ifelse(is.null(ylab), yes = "value", no = ylab)) +
    theme(legend.title = element_text(size = 18),
          legend.text  = element_text(size = 14),
          legend.position = "top",
          axis.text  = element_text(size = 16,colour = "black"),
          axis.title = element_text(size = 16)) +
    guides(fill = guide_legend(title = ifelse(is.null(legendTitle), yes = "Scenario", no = legendTitle)),
           color = guide_legend(title = ifelse(is.null(legendTitle), yes = "Scenario", no = legendTitle)))
}


breedersEq <- function(nSel = 100, varG = 1, h2 = 0.5, nCycPerGen = 1,
                       meanG = 0, nCross = 1000, nProgeny = 1, nQtl = 100, 
                       genLeng = 1, nFounders = 200, nCycle = 10, 
                       altOnly = TRUE, nReps = 3, verbose = T, scenarioName = "Test") {
  # Required packages
  require(AlphaSimR)
  require(dplyr)
  require(ggplot2)
  require(ggpubr)
  
  if (nSel*(nSel-1)/2 < nCross) {
    warning(paste("Only",nSel*(nSel-1)/2,"distinct crosses are possible with",nSel,"individuals"))
  }
  
  # Check if parameters different than default
  params <- c(nSel != 100, varG != 1, h2 != 0.5, nCycPerGen != 1, nProgeny != 1,
              meanG != 0, nCross != 1000, nQtl != 100, genLeng != 1, nFounders != 200)
  alt <- any(params)
  
  # Generate founder haplotypes
  founderPop  <- quickHaplo(nInd = nFounders, nChr = 10, segSites = 2000, genLen = genLeng)
  
  # Set simulation parameters
  SP <- SimParam$new(founderPop)
  
  # Create additive trait
  SP$addTraitA(nQtlPerChr = nQtl, mean = meanG, var = varG)
  
  # Set heritability
  SP$setVarE(h2 = h2)
  
  # Make selection
  output <- data.frame()
  
  for(rep in 1:nReps){
    cat("\n  Working on rep: ", rep, "\n   Cycle")
    pops = vector("list", nCycle+1)
    pops[[1]] = newPop(founderPop, simParam = SP)
    
    for(cycle in 1:nCycle){
      cat(" ", cycle)
      for(gen in 1:nCycPerGen){
        if (gen < nCycPerGen) {
          pops[[cycle]] = selectCross(pop=pops[[cycle]], 
                                      nInd = nSel, nCrosses = nCross, nProgeny = nProgeny,
                                      simParam = SP)
        } else {
          
          pops[[cycle+1]] = selectCross(pop=pops[[cycle]], 
                                        nInd = nSel, nCrosses = nCross, nProgeny = nProgeny,
                                        simParam = SP)
        }
      }
    }
    cat("\n")
    
    # Store
    output = rbind(output,
                   data.frame(cycle    = 0:nCycle,
                              rep      = rep,
                              Scenario = scenarioName,
                              mean     = sapply(pops, meanG),
                              var      = sapply(pops, varG)))
  }
  # Plot 
  plot_mean <- plotOutput(output, "mean", "Genetic Gain")
  plot_var  <- plotOutput(output, "var",  "Genetic Variance")
  
  # Summary of simulation
  if (verbose) {
    cat("\n===========================================")
    cat("\n>>  Simulation parameters")
    cat("\n===========================================")
    cat("\n                                ", scenarioName)
    cat("\nNumber of selected individuals :", ifelse(params[1], paste(nSel,"<--"), nSel))
    cat("\n   Proportion of selected      :", nSel/(nCross*nProgeny))
    cat("\n   Standardised sel. intensity :", round(selInt(nSel/(nCross*nProgeny)),2))
    cat("\nGenetic variance               :", ifelse(params[2], paste(varG,"<--"), varG))
    cat("\nHeritability                   :", ifelse(params[3], paste(h2,"<--"), h2))
    cat("\nGenerations per cycle          :", ifelse(params[4], paste(nCycPerGen,"<--"), nCycPerGen))
    cat("\nTrait mean                     :", ifelse(params[6], paste(meanG,"<--"), meanG))
    cat("\nNumber of selection cycles     :", nCycle)
    cat("\nNumber of crosses per cycle    :", ifelse(params[7], paste(nCross,"<--"), nCross))
    cat("\nNumber of progeny per cross    :", ifelse(params[5], paste(nProgeny,"<--"), nProgeny))
    cat("\nNumber of QTL affecting trait  :", ifelse(params[8], paste(nQtl,"<--"), nQtl))
    cat("\nChromosome genetic length      :", ifelse(params[9], paste(genLeng,"<--"), genLeng))
    cat("\nNumber of founders             :", ifelse(params[10],paste(nFounders,"<--"), nFounders))
    cat("\nNumber of simulation reps      :", nReps)
    cat("\n===========================================\n") 
  }
  
  invisible(list(pop  = pops,
                 data    = output,
                 plot    = ggarrange(plot_mean, plot_var, 
                                     common.legend = TRUE, legend = "top"))
  )
}

#---- Check the content of breeders equation function ----
# Check function's parameters
args(breedersEq)
# Check the default parameters of the simulated population
breedersEq(nReps = 2)


# Pick 3 values for the proportion of selected individuals
sel1 <- breedersEq(nSel = 50,  scenarioName = "5%")$data
sel2 <- breedersEq(nSel = 100, scenarioName = "10%")$data
sel3 <- breedersEq(nSel = 200, scenarioName = "20%")$data
tmp <- rbind(sel1,sel2,sel3)
tmp$Scenario <- recode_factor(tmp$Scenario, `5%` = "5%",`10%` = "10%",`20%` = "20%")
a <- plotOutput(tmp, "mean", "Genetic Gain", 
              legendTitle = "Selection intensity")
b <- plotOutput(tmp, "var",  "Genetic Variance", 
              legendTitle = "Selection intensity")
(p1 <- ggarrange(a,b, common.legend = T))
ggsave(plot = p1, filename ="selInt.png", width = 5, height = 2.5, scale = 1.7)
# Note: You can increase selection intensity by decreasing the number of selected 
# individuals selected or increasing the number of selection candidates.


# Pick 3 values for genetic variance
var1 <- breedersEq(varG = 0.5, scenarioName = "0.5")$data
var2 <- breedersEq(varG = 1, scenarioName = "1")$data
var3 <- breedersEq(varG = 2, scenarioName = "2")$data
a <- plotOutput(rbind(var1,var2,var3), "mean", "Genetic Gain", 
              legendTitle = "Genetic variance")
b <- plotOutput(rbind(var1,var2,var3), "var",  "Genetic Variance", 
              legendTitle = "Genetic variance")
(p2 <- ggarrange(a,b, common.legend = T))
ggsave(plot = p2, filename ="genVar.png", width = 5, height = 2.5, scale = 1.7)


# Pick 3 values for accuracy of selection
hh1 <- breedersEq(h2 = 0.01,  scenarioName = paste(round(sqrt(0.01),1)))$data
hh2 <- breedersEq(h2 = 0.25,  scenarioName = paste(round(sqrt(0.25),1)))$data
hh3 <- breedersEq(h2 = 0.5,  scenarioName = paste(round(sqrt(0.5),1)))$data
hh4 <- breedersEq(h2 = 1, scenarioName = "1 (TGV)")$data
a <- plotOutput(rbind(hh1,hh2,hh3,hh4), "mean", "Genetic Gain", 
              legendTitle = "Accuracy of selection")
b <- plotOutput(rbind(hh1,hh2,hh3,hh4), "var",  "Genetic Variance", 
              legendTitle = "Accuracy of selection")
(p3 <- ggarrange(a,b, common.legend = T))
ggsave(plot = p3, filename ="selAcc.png", width = 5, height = 2.5, scale = 1.7)

# Pick 3 values for generation interval
gen1 <- breedersEq(nCycPerGen = 1,  scenarioName = "1/year")$data
gen2 <- breedersEq(nCycPerGen = 2,  scenarioName = "0.5/year")$data
gen3 <- breedersEq(nCycPerGen = 4,  scenarioName = "0.25/year")$data
a <- plotOutput(rbind(gen1,gen2,gen3), "mean", "Genetic Gain", 
         legendTitle = "Generation interval")
b <- plotOutput(rbind(gen1,gen2,gen3), "var",  "Genetic Variance", 
         legendTitle = "Generation interval")
(p4 <- ggarrange(a,b, common.legend = T))
ggsave(plot = p4, filename ="genInt.png", width = 5, height = 2.5, scale = 1.7)
# Conclusion: response to selection can be increased by decreasing generation interval


# Plot all components of breeders equation together
#====================================================================
ggarrange(p1,p2,p3,p4, nrow = 4)




# Try other parameters in the function and see how they impact 
# long term genetic gain and variance 
#====================================================================
# Number of QTLs
qtl1 <- breedersEq(nQtl = 1,    scenarioName = "1")$data
qtl2 <- breedersEq(nQtl = 15,   scenarioName = "20")$data
qtl3 <- breedersEq(nQtl = 200,  scenarioName = "200")$data
a <- plotOutput(rbind(qtl1,qtl2,qtl3), "mean", "Genetic Gain", 
         legendTitle = "Number of QTLs")
b <- plotOutput(rbind(qtl1,qtl2,qtl3), "var",  "Genetic Variance", 
         legendTitle = "Number of QTLs")
(p5 <- ggarrange(a,b, common.legend = T))

# Map genetic length
# Note: Recombination rate is increased by increasing the length
# of the genetic map.
lng1 <- breedersEq(genLeng = 1, scenarioName = "1")$data
lng2 <- breedersEq(genLeng = 3, scenarioName = "3")$data
lng3 <- breedersEq(genLeng = 30, scenarioName = "30")$data
a <- plotOutput(rbind(lng1,lng2,lng3), "mean", "Genetic Gain", 
         legendTitle = "Genetic length")
b <- plotOutput(rbind(lng1,lng2,lng3), "var",  "Genetic Variance", 
         legendTitle = "Genetic length")
(p6 <- ggarrange(a,b, common.legend = T))


# Test the effect of number of progeny per cross 
# Also inspect whether it is better to make more crosses and fewer progeny or vice versa
prog0 <- breedersEq(nCross = 30, nProgeny = 1,   nReps = 5, scenarioName = "RM")$data
prog1 <- breedersEq(nCross = 30, nProgeny = 5,   nReps = 5, scenarioName = "30-5")$data
prog2 <- breedersEq(nCross = 30, nProgeny = 50,  nReps = 5, scenarioName = "30-50")$data
prog3 <- breedersEq(nCross = 30, nProgeny = 100, nReps = 5, scenarioName = "30-100")$data
prog4 <- breedersEq(nCross = 100, nProgeny = 30, nReps = 5, scenarioName = "100-30")$data
prog5 <- breedersEq(nCross = 200, nProgeny = 15, nReps = 5, scenarioName = "200-15")$data
prog6 <- breedersEq(nCross = 300, nProgeny = 10, nReps = 5, scenarioName = "300-10")$data
a <- plotOutput(rbind(prog0,prog1,prog2,prog3,prog4,prog5,prog6), "mean", "Genetic Gain", 
           legendTitle = "Number of progeny/cross")
b <- plotOutput(rbind(prog0,prog1,prog2,prog3,prog4,prog5,prog6), "var",  "Genetic Variance", 
           legendTitle = "Number of progeny/cross")
ggarrange(a,b, common.legend = T)

